<?php
namespace Drupal\pagedesigner_parts\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\node\Entity\Node;
use Drupal\pagedesigner\Service\Renderer;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Pagedesigner block
 *
 * @Block(
 *   id = "pagedesigner_part",
 *   admin_label = @Translation("Pagedesigner part"),
 * )
 */

class PagedesignerPart extends BlockBase implements ContainerFactoryPluginInterface
{

    protected $node = null;

    /**
     * Undocumented variable
     *
     * @var [type]
     */
    protected $renderer = null;

    /**
     * @param array $configuration
     * @param string $plugin_id
     * @param mixed $plugin_definition
     * @param Renderer $renderer
     */
    public function __construct(array $configuration, $plugin_id, $plugin_definition, Renderer $renderer)
    {
        parent::__construct($configuration, $plugin_id, $plugin_definition);
        $this->renderer = $renderer;
    }

    /**
     * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
     * @param array $configuration
     * @param string $plugin_id
     * @param mixed $plugin_definition
     * @return static
     */
    public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
    {
        return new static(
            $configuration,
            $plugin_id,
            $plugin_definition,
            $container->get('pagedesigner.service.renderer')
        );
    }

    public function build()
    {
        $build = array(
            '#theme' => 'pagedesigner-part',
            '#markup' => '',
            '#cache' => ['max-age' => 0],
        );

        $config = $this->getConfiguration();
        $assignedNode = -1;
        if (isset($config['assignedNode'])) {
            $assignedNode = $config['assignedNode'];
            $nodeId = \Drupal::routeMatch()->getRawParameter('node');
            if ($nodeId != $assignedNode) {
                $this->node = Node::load($assignedNode);
                if ($this->node != null) {
                    $language = \Drupal::service('language_manager')->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
                    if ($this->node->hasTranslation($language)) {
                        $this->node = $this->node->getTranslation($language);
                        $container = $this->node->field_pagedesigner_content->entity;
                        if ($container != null) {
                            if (!\Drupal::currentUser()->isAnonymous()) {
                                $this->renderer->render($container, $this->node);
                            } else {
                                $this->renderer->renderForPublic($container, $this->node);
                            }
                            $build['#markup'] = $this->renderer->getMarkup($this->node);
                            $build['#attached']['html_head']['pagedesigner_dynamic_css' . \time()] = [
                                [
                                    '#tag' => 'style',
                                    '#value' => $this->renderer->getStyles(),
                                    '#attributes' => [
                                        'id' => 'pagedesigner_dynamic_css' . \time(),
                                    ],
                                ],
                                'pagedesigner_dynamic_css' . \time(),
                            ];
                            $build['#cache'] = ['keys' =>
                                ['entity_view', 'node', $this->node->id()],
                                'contexts' => ['languages'],
                                'tags' => $this->node->getCacheTags(),
                                'max-age' => Cache::PERMANENT,
                            ];
                        }
                    }
                }
            }
        }
        return $build;
    }

    /**
     * {@inheritdoc}
     */
    public function blockForm($form, FormStateInterface $form_state)
    {
        $form = parent::blockForm($form, $form_state);
        $node = null;
        $assignedNode = -1;
        if (isset($this->configuration['assignedNode'])) {
            $assignedNode = $this->configuration['assignedNode'];
            $node = Node::load($assignedNode);
        }
        $form['assignedNode'] = array(
            '#type' => 'entity_autocomplete',
            '#title' => $this->t('Assigned part'),
            '#target_type' => 'node',
            '#selection_settings' => ['target_bundles' => ['pagedesigner_part']],
            '#tags' => true,
            '#size' => 30,
            '#maxlength' => 1024,
            '#default_value' => $node,
            '#required' => true,
        );
        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function blockSubmit($form, FormStateInterface $form_state)
    {
        $this->configuration['assignedNode'] = $form_state->getValue('assignedNode')[0]["target_id"];
    }

    /**
     * {@inheritdoc}
     */
    public function getCacheTags()
    {
        if ($this->node != null) {
            return Cache::mergeTags(parent::getCacheTags(), $this->node->getCacheTags());
        }
        return parent::getCacheTags();
    }

}
